Package: ompr
Type: Package
Title: Model and Solve Mixed Integer Linear Programs
Version: 1.0.4
Authors@R: person("Dirk", "Schumacher", email = "mail@dirk-schumacher.net",
            role = c("aut", "cre"))
Description: Model mixed integer linear programs in an algebraic way directly in R.
             The model is solver-independent and thus offers the possibility
             to solve a model with different solvers. It currently only supports
             linear constraints and objective functions. See the 'ompr'
             website <https://dirkschumacher.github.io/ompr/> for more information,
             documentation and examples.
License: MIT + file LICENSE
RoxygenNote: 7.2.3
Encoding: UTF-8
URL: https://github.com/dirkschumacher/ompr
BugReports: https://github.com/dirkschumacher/ompr/issues
Depends: R (>= 3.5.0)
Imports: lazyeval, rlang (>= 0.2.0), listcomp (>= 0.4.0), methods,
        data.table, Matrix, fastmap
Suggests: covr, magrittr, testthat
ByteCompile: Yes
Collate: 'abstract-model-impl.R' 'helper.R'
        'linear-optimization-model-impl.R'
        'linear-optimization-model-linear-constraints.R'
        'linear-optimization-model-linear-functions.R' 'model-api.R'
        'milp-impl.R' 'milp-linearopt-variables.R' 'ompr-package.R'
        'solution-api.R' 'solution-impl.R'
NeedsCompilation: no
Packaged: 2023-09-09 06:12:03 UTC; dsp
Author: Dirk Schumacher [aut, cre]
Maintainer: Dirk Schumacher <mail@dirk-schumacher.net>
Repository: CRAN
Date/Publication: 2023-09-09 08:40:02 UTC
Built: R 4.4.0; ; 2024-04-29 22:02:48 UTC; unix
