# Generated by roxygen2: do not edit by hand

S3method("*",AbstractLinearFunction)
S3method("+",AbstractLinearFunction)
S3method("-",AbstractLinearFunction)
S3method("/",AbstractLinearFunction)
S3method("<=",AbstractLinearFunction)
S3method("==",AbstractLinearFunction)
S3method(">=",AbstractLinearFunction)
S3method("[",OmprLinearVariableCollection)
S3method(add_constraint,abstract_model)
S3method(add_constraint,linear_optimization_model)
S3method(add_constraint_,linear_optimization_model)
S3method(add_constraint_,milp_model)
S3method(add_variable,abstract_model)
S3method(add_variable,linear_optimization_model)
S3method(add_variable_,linear_optimization_model)
S3method(add_variable_,milp_model)
S3method(additional_solver_output,solution)
S3method(extract_bound_from_constraint,LinearFunction)
S3method(extract_bound_from_constraint,LinearTerm)
S3method(extract_constraints,linear_optimization_model)
S3method(extract_constraints,milp_model)
S3method(flip_constaint_sense,LinearConstraintSenseEq)
S3method(flip_constaint_sense,LinearConstraintSenseGeq)
S3method(flip_constaint_sense,LinearConstraintSenseLeq)
S3method(get_column_duals,solution)
S3method(get_row_duals,solution)
S3method(get_solution,solution)
S3method(n_columns,LinearTerm)
S3method(n_columns,OmprLinearVariable)
S3method(n_columns,OmprLinearVariableCollection)
S3method(nconstraints,linear_optimization_model)
S3method(nconstraints,milp_model)
S3method(new_linear_constraint,LinearFunction)
S3method(new_linear_constraint,LinearTerm)
S3method(nvars,linear_optimization_model)
S3method(nvars,milp_model)
S3method(objective_function,linear_optimization_model)
S3method(objective_function,milp_model)
S3method(objective_value,solution)
S3method(print,linear_optimization_model)
S3method(print,milp_model)
S3method(print,solution)
S3method(set_bounds,abstract_model)
S3method(set_bounds,linear_optimization_model)
S3method(set_bounds_,linear_optimization_model)
S3method(set_bounds_,milp_model)
S3method(set_objective,abstract_model)
S3method(set_objective,linear_optimization_model)
S3method(set_objective_,linear_optimization_model)
S3method(set_objective_,milp_model)
S3method(solve_model,abstract_model)
S3method(solver_status,solution)
S3method(variable_bounds,linear_optimization_model)
S3method(variable_bounds,milp_model)
S3method(variable_keys,linear_optimization_model)
S3method(variable_keys,milp_model)
S3method(variable_types,linear_optimization_model)
S3method(variable_types,milp_model)
export(MILPModel)
export(MIPModel)
export(add_constraint)
export(add_constraint_)
export(add_variable)
export(add_variable_)
export(additional_solver_output)
export(as_colwise)
export(colwise)
export(extract_constraints)
export(get_column_duals)
export(get_row_duals)
export(get_solution)
export(get_solution_)
export(nconstraints)
export(new_solution)
export(nvars)
export(objective_function)
export(objective_value)
export(set_bounds)
export(set_bounds_)
export(set_objective)
export(set_objective_)
export(solve_model)
export(solver_status)
export(sum_expr)
export(sum_over)
export(variable_bounds)
export(variable_keys)
export(variable_types)
import(data.table)
importFrom(Matrix,sparseMatrix)
importFrom(Matrix,sparseVector)
importFrom(fastmap,fastmap)
importFrom(lazyeval,as.lazy)
importFrom(lazyeval,as.lazy_dots)
importFrom(lazyeval,lazy_dots)
importFrom(listcomp,gen_list)
importFrom(methods,new)
importFrom(rlang,abort)
importFrom(rlang,caller_env)
importFrom(rlang,enquo)
importFrom(rlang,enquos)
importFrom(rlang,eval_bare)
importFrom(rlang,get_expr)
importFrom(rlang,new_environment)
importFrom(rlang,new_quosure)
importFrom(rlang,syms)
importFrom(stats,setNames)
